-- Representation and display as dot of timed automata

module TimedAutomaton (
   Clock(..), ClockConstraint(..), Location(..), Label(..), Transition(..), Automaton(..),
   maxConstant, showInfo, getClockName
) where

import Data.List

data Clock = Clock String
data ClockConstraint = LowerThan (Clock,Int) | GreaterThan (Clock,Int) | EqualTo (Clock, Int)
data Location = Location String
data Label = Label String
data Transition = Transition { source::Location, guard::[ClockConstraint], label::Label, resets::[Clock], target::Location }
data Automaton = Automaton { initial::Location, goals::[Location], clocks::[Clock], transitions::[Transition] }

instance Eq Clock where
   Clock i == Clock i' = i == i'

instance Ord Clock where
   compare (Clock i) (Clock i') = compare i i' 

instance Show Clock where
   show (Clock i) = i

instance Eq ClockConstraint where
   LowerThan (c,i) == LowerThan (c',i') = c == c' && i == i'
   GreaterThan (c, i) == GreaterThan (c',i') = c == c' && i == i'
   EqualTo (c,i) == EqualTo (c',i') = c == c' && i == i'
   _ == _ = False

instance Show Location where
   show (Location s) = s

instance Eq Location where
   Location s == Location s' = s == s'

instance Show Label where
   show (Label s) = s

instance Eq Label where
   Label s == Label s' = s == s'

instance Show Transition where
   show t = "(" ++ (show (source t)) ++ ", " ++ (show (label t)) ++ ", " ++ (show (target t))

instance Show Automaton where
   show a = "digraph A {\n" ++
            (showMarked (initial a) (goals a)) ++
            (concatMap (showTransition) (transitions a)) ++ "}"

showTransition :: Transition -> String
showTransition t = (showLocation $ source t) ++ " -> " ++ 
                   (showLocation $ target t) ++ " [label=\"" ++  
                   (showInfo (guard t) (label t) (resets t)) ++ "\"];\n"

showLocation :: Location -> String
showLocation (Location l) = l

showInfo :: [ClockConstraint] -> Label -> [Clock] -> String
showInfo ccs (Label l) cs = (concatMap (\x -> (showConstraint x) ++ ", ") ccs) ++ 
                             l ++
                             (concatMap (showReset) cs) 

showConstraint :: ClockConstraint -> String
showConstraint (LowerThan (c,i)) = (showClock c) ++ " < " ++ (show i)
showConstraint (GreaterThan (c,i)) = (showClock c) ++ " > " ++ (show i)
showConstraint (EqualTo (c,i)) = (showClock c) ++ " = " ++ (show i)

showReset :: Clock -> String
showReset c = ", " ++ (showClock c) ++ " := 0"

showClock :: Clock -> String
showClock (Clock c) = c

showMarked :: Location -> [Location] -> String
showMarked (Location l) [] = "dummy [style=invis,shape=none];\n" ++ "dummy -> " ++ l ++ "\n"
showMarked (Location l) ((Location l'):locs) 
   | l' == l =  "dummy [style=invis,shape=none];\n" ++ 
                l ++ " [peripheries=2];\n" ++
                "dummy -> " ++ l ++ ";\n" ++ 
                (showMarked' locs)
   | otherwise = l' ++ " [peripheries=2];\n" ++ (showMarked (Location l) locs)
showMarked' [] = "";
showMarked' ((Location l):locs) = l ++ " [peripheries=2];\n" ++ (showMarked' locs)

-- Give the maximum constant in the guard of an automaton
maxConstant :: Automaton -> Int
maxConstant a = last $ sort (map getConstant (concatMap guard (transitions a)))

-- Get the constant in a constraint
getConstant :: ClockConstraint -> Int
getConstant (LowerThan (_,i)) = i
getConstant (GreaterThan (_,i)) = i
getConstant (EqualTo (_,i)) = i

-- Get the id of a clock
getClockName :: Clock -> String
getClockName (Clock c) = c